@@ -41,6 +41,14 @@ namespace {
     // we'll find either std::abs or else another version via ADL:
     return abs (t);
   }
+  // unsigned types are always non-negative
+  template<> unsigned int boost_numeric_ublas_abs (const unsigned int& t) {
+    return t;
+  }
+  // unsigned types are always non-negative
+  template<> unsigned long boost_numeric_ublas_abs (const unsigned long& t) {
+    return t;
+  }
 }
 
 namespace boost { namespace numeric { namespace ublas {
@@ -594,10 +602,10 @@ namespace boost { namespace numeric { namespace ublas {
         struct has_trivial_destructor : public boost::has_trivial_destructor<T> {};
 
         template<typename FLT>
-        struct has_trivial_constructor<std::complex<FLT> > : public boost::true_type {};
+        struct has_trivial_constructor<std::complex<FLT> > : public has_trivial_constructor<FLT> {};
         
         template<typename FLT>
-        struct has_trivial_destructor<std::complex<FLT> > : public boost::true_type {};
+        struct has_trivial_destructor<std::complex<FLT> > : public has_trivial_destructor<FLT> {};
 
     }
 
